package com.qmth.digit.print.business.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.qmth.digit.print.business.bean.dto.CardRuleDto;
import com.qmth.digit.print.business.bean.query.BlurryParam;
import com.qmth.digit.print.business.bean.query.CardRuleParam;
import com.qmth.digit.print.business.entity.BasicCardRule;
import com.qmth.digit.print.business.entity.SysOrg;
import com.qmth.digit.print.business.mapper.BasicCardRuleMapper;
import com.qmth.digit.print.business.service.BasicCardRuleService;
import com.qmth.digit.print.business.service.BasicTemplateOrgService;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import com.qmth.digit.print.common.enums.TemplateTypeEnum;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Objects;

/**
 * <p>
 * 题卡规则 服务实现类
 * </p>
 *
 * @author xf
 * @since 2021-03-23
 */
@Service
public class BasicCardRuleServiceImpl extends ServiceImpl<BasicCardRuleMapper, BasicCardRule> implements BasicCardRuleService {

    @Autowired
    private BasicTemplateOrgService basicTemplateOrgService;

    @Override
    public IPage<CardRuleDto> list(CardRuleParam param) {
        Page<CardRuleDto> page = new Page<>(param.getPageNumber(), param.getPageSize());
        IPage<CardRuleDto> cardRuleDtoIPage = this.baseMapper.listPage(page, param);
        if (cardRuleDtoIPage.getRecords().size() > 0) {
            cardRuleDtoIPage.getRecords().forEach(m -> {
                //查询适用学院
                List<SysOrg> sysOrgs = basicTemplateOrgService.listByTypeAndTemplateId(TemplateTypeEnum.CARD_RULE.name(), m.getId());
                m.setOrgs(sysOrgs);
            });
        }
        return cardRuleDtoIPage;
    }

    @Override
    public List<BasicCardRule> list(BlurryParam param) {
        QueryWrapper<BasicCardRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCardRule::getSchoolId, param.getSchoolId()).like(BasicCardRule::getName, param.getParam());
        return this.list(queryWrapper);
    }

    @Override
    public boolean enable(BasicCardRule cardRule) {
        // todo 校验规则是否有使用
        UpdateWrapper<BasicCardRule> updateWrapper = new UpdateWrapper<>();
        updateWrapper.lambda().set(BasicCardRule::getEnable, cardRule.getEnable()).eq(BasicCardRule::getId, cardRule.getId());
        return this.update(updateWrapper);
    }

    @Transactional
    @Override
    public boolean saveCardRule(BasicCardRule cardRule) {
        // todo 校验是否有必选字段已使用
        QueryWrapper<BasicCardRule> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(BasicCardRule::getSchoolId, cardRule.getSchoolId()).eq(BasicCardRule::getName, cardRule.getName());
        BasicCardRule basicCardRule = this.getOne(queryWrapper);
        boolean isSuccess;
        // 新增
        if (Objects.isNull(basicCardRule.getId())) {
            if (basicCardRule != null) {
                throw ExceptionResultEnum.ERROR.exception("题卡规则名称已存在");
            }
            cardRule.setCreateTime(System.currentTimeMillis());
            isSuccess = this.save(cardRule);
        }
        // 修改
        else {
            if (basicCardRule != null && basicCardRule.getId() != cardRule.getId()) {
                throw ExceptionResultEnum.ERROR.exception("题卡规则名称已存在");
            }

            cardRule.setUpdateTime(System.currentTimeMillis());

            //删除权限
            basicTemplateOrgService.removeByRuleId(cardRule.getId());
            isSuccess = this.updateById(cardRule);

        }

        // 新增权限
        basicTemplateOrgService.saveBatch(cardRule.getId(), cardRule.getOrgIds(), cardRule.getCreateId());

        return isSuccess;
    }
}
