package com.qmth.digit.print.api;

import cn.hutool.core.lang.UUID;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.qmth.boot.api.annotation.Aac;
import com.qmth.boot.api.annotation.BOOL;
import com.qmth.boot.api.constant.ApiConstant;
import com.qmth.digit.print.business.bean.query.LoginParam;
import com.qmth.digit.print.business.config.DictionaryConfig;
import com.qmth.digit.print.business.entity.BasicVerifyCode;
import com.qmth.digit.print.business.entity.SysUser;
import com.qmth.digit.print.business.service.BasicVerifyCodeService;
import com.qmth.digit.print.business.service.SysConfigService;
import com.qmth.digit.print.business.service.SysUserService;
import com.qmth.digit.print.business.util.security.Md5Utils;
import com.qmth.digit.print.common.enums.ExceptionResultEnum;
import com.qmth.digit.print.common.util.Result;
import com.qmth.digit.print.common.util.ResultUtil;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;

import java.util.Date;
import java.util.Objects;

/**
 * @Date: 2021/3/30.
 */
@Api(tags = "系统Controller")
@RestController
@RequestMapping(ApiConstant.DEFAULT_URI_PREFIX + "/${prefix.url.common}")
@Aac(auth = BOOL.FALSE)
public class SysController {

    @Autowired
    private SysUserService sysUserService;

    @Autowired
    private BasicVerifyCodeService basicVerifyCodeService;

    @Autowired
    private SysConfigService sysConfigService;

    @Autowired
    private DictionaryConfig dictionaryConfig;

    @ApiOperation(value = "登录")
    @RequestMapping(value = "/login", method = RequestMethod.POST)
    public Result login(@RequestBody LoginParam login) {
        QueryWrapper<SysUser> queryWrapper = new QueryWrapper<>();
        queryWrapper.lambda().eq(SysUser::getLoginName, login.getLoginName());
        SysUser sysUser = sysUserService.getOne(queryWrapper);
        if (sysUser == null) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        String password = Md5Utils.toMd5Hex(login.getPassword());
        if (password.equals(sysUser.getPassword())) {
            throw ExceptionResultEnum.ERROR.exception("密码错误");
        }
        //校验验证码
        String value = sysConfigService.getByKey("sys.code.enable");
        if (value.equals("true")) {
            String code = login.getCode();
            if (Objects.isNull(code)) {
                throw ExceptionResultEnum.ERROR.exception("验证码为空");
            }
            if (!dictionaryConfig.smsDomain().getSmsNormalCode().equals(code)) {
                QueryWrapper<BasicVerifyCode> codeWrapper = new QueryWrapper<>();
                codeWrapper.lambda().eq(BasicVerifyCode::getMobileNumber, sysUser.getMobileNumber()).eq(BasicVerifyCode::getUserId, sysUser.getId());
                BasicVerifyCode accessControl = basicVerifyCodeService.getOne(codeWrapper);
                if (accessControl == null || (accessControl != null && !accessControl.getVerifyCode().equals(code))) {
                    throw ExceptionResultEnum.ERROR.exception("短信验证码错误，请仔细核对后再次输入");
                }

                if (new Date(accessControl.getExpireTime()).before(new Date())) {
                    throw ExceptionResultEnum.ERROR.exception("短信验证码已过期");
                }
            }
        }

        //停用
        if (!sysUser.getEnable()) {
            throw ExceptionResultEnum.ERROR.exception("用户已禁用");
        }
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("user", sysUser);
        jsonObject.put("token", UUID.fastUUID().toString());
        return ResultUtil.ok(jsonObject);
    }

    /**
     * 发送验证码
     * @param loginParam
     * @return
     */
    @ApiOperation(value = "发送验证码")
    @RequestMapping(value = "/getVerifyCode", method = RequestMethod.POST)
    public Object getverifyCode(@RequestBody LoginParam loginParam) {
        String loginName = loginParam.getLoginName();
        QueryWrapper<SysUser> wrapper = new QueryWrapper<>();
        wrapper.lambda().eq(SysUser::getLoginName, loginName);
        SysUser user = sysUserService.getOne(wrapper);
        //用户不存在
        if (Objects.isNull(user)) {
            throw ExceptionResultEnum.ERROR.exception("用户不存在");
        }
        String password = Md5Utils.toMd5Hex(loginParam.getPassword());
        if (password.equals(user.getPassword())) {
            throw ExceptionResultEnum.ERROR.exception("密码错误");
        }
        String mobileNumber = user.getMobileNumber();
        if (Objects.isNull(mobileNumber)) {
            throw ExceptionResultEnum.ERROR.exception("用户未绑定手机号码");
        }
        basicVerifyCodeService.sendVeirfyCode(mobileNumber, user.getId());
        return ResultUtil.ok(true);
    }

}
